import './style.css'
import Contextmenu from './content'
import {createDom} from './utils'
import { getValue } from '../../utils';

export default {
  name: 'LxBreadcrumb',
  props: {
    routes: {
      type: Array,
      default() {
        return []
      }
    },
    primaryKey: {
      type: String,
      default: 'id'
    },
    activeRouter: {
      type: Object,
      default () {
        return {}
      }
    },
    activeValue: {
      type: [String, Number],
      default: ''
    }
  },
  data() {
    return {
      tabList: [],
      showContentMenu: false,
      contentMenu: {
        left: '0px',
        top: '0px'
      },
      contextMenuInstance: null
    }
  },
  methods: {
    handlerListWidth(list) {
      let sumWidth = list.reduce((start, item) => {
        if (item.elm) {
          start += 150
          return start
        }
      }, 0)
      sumWidth += (list.length - 1) * 5
      this.$refs.$$lxBreadcrumb.style.width = sumWidth + 'px'
    },
    createWrap(h) {
      return h('div', {
        class: ['lx__breadcrumb_wrap']
      }, [this.createContent(h)])
    },
    createContent(h) {
      const tabVNList = this.createItemList(h)
      this.$nextTick(() => {
        this.handlerListWidth(tabVNList)
      })
      return h('div', {
        class: ['lx__breadcrumb_list'],
        ref: '$$lxBreadcrumb'
      }, tabVNList)
    },
    createItemList(h) {
      const list = []

      for (let i = 0; i < this.routes.length; i++) {
        list.push(this.createItem(h, this.routes[i], i))
      }
      return list
    },
    createItem(h, item, index) {
      return h('div', {
        class: ['lx__breadcrumb_item'],
      }, [this.createItemContent(h, item, index)])
    },
    createItemContent(h, item, index) {
      const active = getValue(this.activeRouter, this.primaryKey) === getValue(item, this.primaryKey)

      return h('div', {
        class: ['item__content', active && 'breadcrumb_active'],
        on: {
          contextmenu: (e) => {
            this.handlerRightKey(e, item, index)
          }
        }
      }, [this.createLabel(h, item), this.createClose(h, item, index)])
    },
    createLabel(h, item) {
      const {meta, fullPath} = item
      return h('div', {
        class: ['item__label'],
        attrs: {
          title: meta.title
        },
        on: {
          click: () => {
            this.$router.push(fullPath)
          }
        }
      }, meta.title)
    },
    createClose(h, item, index) {
      return h('div', {
        class: ['lx__breadcrumb_close'],
        on: {
          click: (e) => {
            this.$emit('delete', this.activeRouter, item, index)
          }
        },
        attrs: {
          title: '关闭'
        }
      }, [
        h('i', {
          class: ['el-icon-close']
        })
      ])
    },
    handlerRightKey(el, item, index) {
      el.preventDefault()
      this.createContentMenu(el, item, index)
    },
    createContentMenu(el, item, index) {
      const {clientX, clientY} = el
      this.contentMenu.left = clientX + 'px'
      this.contentMenu.top = clientY + 'px'
      if (this.contextMenuInstance) {
        this.contextMenuInstance.remove()
      }
      this.contextMenuInstance = createDom(
        Contextmenu,
        {
          props: {
            show: this.showContentMenu,
            left: this.contentMenu.left,
            top: this.contentMenu.top
          },
          on: {
            click: (selected) => {
              this.$emit('select-menu', {
                selected,
                currentRouter: this.activeRouter,
                item,
                index
              })
            }
          }
        },
        () => {
          this.contextMenuInstance = null
        }
      )
    },
    initActive() {

    }
  },
  render(h) {
    return this.createWrap(h)
  },
  created() {

  },
  mounted() {
  }
}
