import { TimePicker, DatePicker, TimeSelect } from 'element-ui'
import {isUndefined, isArray, isDate, getDateTime} from '../../../utils';

// 默认时间选择
export function DefaultTimeSelect (h, item, renderOptions) {
  return returnVN(h, TimeSelect, renderOptions)
}
// 默认日期选择
export function DefaultDatePicker (h, item, renderOptions) {
  return returnVN(h, DatePicker, renderOptions)
}
// 单个时间
export function handlerTime (h, item, renderOptions) {
  const { props } = renderOptions
  const { value } = props
  if (isUndefined(value)) {
    props.value = null
  }
  if (value) {
    if (!isDate(value)) {
      const { year, month, date } = getDateTime()
      const DATE = `${year}-${month}-${date}`
      props.value = new Date(`${DATE} ${value}`)
    }
  }

  return returnVN(h, TimePicker, renderOptions)
}
// 时间范围
export function handlerTimeRange (h, item, renderOptions) {
  const { props } = renderOptions
  const { value } = props
  if (isUndefined(value) || !isArray(value) || value.length < 1) {
    props.value = null
  }

  if (isArray(value) && value.length) {
    if (!isDate(value[0]) && !isDate(value[1])) {
      const { year, month, date } = getDateTime()
      const DATE = `${year}-${month}-${date}`
      value[0] = new Date(`${DATE} ${value[0]}`)
      value[1] = new Date(`${DATE} ${value[1]}`)
      props.value = value
    }
  }

  const options = {
    ...renderOptions,
    props: {
      isRange: true,
      rangeSeparator: '至',
      startPlaceholder: '开始时间',
      endPlaceholder: '截止时间',
      ...props
    }
  }

  return returnVN(h, TimePicker, options)
}
// 日期
export function handlerDate (h, item, renderOptions) {
  const pickerOptions = {
    // disabledDate(time) {
    //   return time.getTime() > Date.now();
    // },
    shortcuts: [{
      text: '今天',
      onClick(picker) {
        picker.$emit('pick', new Date());
      }
    }, {
      text: '昨天',
      onClick(picker) {
        const date = new Date();
        date.setTime(date.getTime() - 3600 * 1000 * 24);
        picker.$emit('pick', date);
      }
    }, {
      text: '一周前',
      onClick(picker) {
        const date = new Date();
        date.setTime(date.getTime() - 3600 * 1000 * 24 * 7);
        picker.$emit('pick', date);
      }
    }]
  }

  const { props } = renderOptions
  const options = {
    ...renderOptions,
    props: {
      pickerOptions: pickerOptions,
      ...props
    }
  }

  return returnVN(h, DatePicker, options)
}
// 日期范围
export function handlerDateRange (h, item, renderOptions) {
  const pickerOptions = {
    shortcuts: [{
      text: '最近一周',
      onClick(picker) {
        const end = new Date();
        const start = new Date();
        start.setTime(start.getTime() - 3600 * 1000 * 24 * 7);
        picker.$emit('pick', [start, end]);
      }
    }, {
      text: '最近一个月',
      onClick(picker) {
        const end = new Date();
        const start = new Date();
        start.setTime(start.getTime() - 3600 * 1000 * 24 * 30);
        picker.$emit('pick', [start, end]);
      }
    }, {
      text: '最近三个月',
      onClick(picker) {
        const end = new Date();
        const start = new Date();
        start.setTime(start.getTime() - 3600 * 1000 * 24 * 90);
        picker.$emit('pick', [start, end]);
      }
    }]
  }
  const { props } = renderOptions
  const options = {
    ...renderOptions,
    props: {
      type: 'daterange',
      pickerOptions: pickerOptions,
      unlinkPanels: true,
      rangeSeparator: '至',
      startPlaceholder: '开始时间',
      endPlaceholder: '截止时间',
      ...props
    }
  }

  return returnVN(h, DatePicker, options)
}
// 日期时间
export function handlerDateTime (h, item, renderOptions) {
  const pickerOptions = {
    shortcuts: [{
      text: '今天',
      onClick(picker) {
        picker.$emit('pick', new Date());
      }
    }, {
      text: '昨天',
      onClick(picker) {
        const date = new Date();
        date.setTime(date.getTime() - 3600 * 1000 * 24);
        picker.$emit('pick', date);
      }
    }, {
      text: '一周前',
      onClick(picker) {
        const date = new Date();
        date.setTime(date.getTime() - 3600 * 1000 * 24 * 7);
        picker.$emit('pick', date);
      }
    }]
  }

  const { props } = renderOptions
  const options = {
    ...renderOptions,
    props: {
      type: 'datetime',
      pickerOptions: pickerOptions,
      ...props
    }
  }

  return returnVN(h, DatePicker, options)
}
// 日期时间范围
export function handlerDateTimeRange (h, item, renderOptions) {
  const pickerOptions = {
    shortcuts: [{
      text: '最近一周',
      onClick(picker) {
        const end = new Date();
        const start = new Date();
        start.setTime(start.getTime() - 3600 * 1000 * 24 * 7);
        picker.$emit('pick', [start, end]);
      }
    }, {
      text: '最近一个月',
      onClick(picker) {
        const end = new Date();
        const start = new Date();
        start.setTime(start.getTime() - 3600 * 1000 * 24 * 30);
        picker.$emit('pick', [start, end]);
      }
    }, {
      text: '最近三个月',
      onClick(picker) {
        const end = new Date();
        const start = new Date();
        start.setTime(start.getTime() - 3600 * 1000 * 24 * 90);
        picker.$emit('pick', [start, end]);
      }
    }]
  }
  const { props } = renderOptions
  const options = {
    ...renderOptions,
    props: {
      type: 'datetimerange',
      pickerOptions: pickerOptions,
      rangeSeparator: '至',
      startPlaceholder: '开始时间',
      endPlaceholder: '截止时间',
      ...props
    }
  }

  return returnVN(h, DatePicker, options)
}

function returnVN (h, component, options) {
  if (options.props.isEdit) {
    return h(component, options)
  }
  options.props.disabled = !options.props.isEdit
  return h(component, options)
}
