import { InputNumber } from 'element-ui';

/**
 * 处理 input-number 列表
 * @param h form组件的createElement
 * @param item 当前项的表单数据
 * @param renderOptions 整理好的表单属性
 * */
export function handlerInputNumberItem (h, item, renderOptions) {
  let leftText, rightText
  if (item.componentProps) {
    leftText = item.componentProps.leftText
    rightText = item.componentProps.rightText
  }

  const { value, isEdit } = renderOptions.props
  let childrenVN = []
  if (leftText) {
    childrenVN.push(handlerLRText(h, leftText, 'left'))
  }
  let VNode
  if (isEdit) {
    VNode = h(InputNumber, renderOptions)
  } else {
    VNode = h('span', value || '-')
  }

  childrenVN.push(VNode)

  if (rightText) {
    childrenVN.push(handlerLRText(h, rightText, 'right'))
  }

  return childrenVN
}

/**
 * 左右的文字
 * @param h createElement
 * @param option 配置
 * @param type 类型 left right
 * @returns VNode
 * */
export function handlerLRText (h, option, type) {
  let VN = null

  const margin = 'margin' + (type === 'left' ? 'Right' : 'Left')

  if (typeof option === 'string' || typeof option === 'number') {
    VN = h('span', {
      style: {
        [margin]: '5px'
      }
    }, option)
  }
  if (typeof option === 'object') {
    VN = h('span', {
      style: {
        [margin]: '5px',
        ...option.style
      }
    }, option.text || '')
  }
  return VN
}
