import { get, set } from "lodash";
import { Radio, RadioButton, RadioGroup} from "element-ui";
import {effectiveValue} from '../../../utils';
import './style.css'

/**
 * 处理 radio 单选组件
 * @param h form createElement
 * @param item 当前项的表单数据
 * @param renderOptions 整理好的表单属性
 * */
export function handlerRadioItem (h, item, renderOptions) {
  return handlerRadioBase.call(this, Radio, { h, item, renderOptions })
}
export function handlerRadioButtonItem (h, item, renderOptions) {
  return handlerRadioBase.call(this, RadioButton, { h, item, renderOptions })
}
export function handlerRadioBase (component, { h, item, renderOptions }) {
  const fm = this
  const { options, optionsConfig = [], filed, on = {} } = item
  const { value, isEdit } = renderOptions.props
  if (!isEdit) {
    const content = []
    if (value && value.length) {
      for (let i = 0; i < options.length; i++) {
        const item = options[i]
        if (value.includes(item.value)) {
          content.push(item.label)
        }
      }
    }
    return h('span', null, content.toString())
  }

  const radios = options.map((item, index) => {
    let radioProps = {
      props: {
        label: item.value
      }
    }
    if (optionsConfig[index]) {
      radioProps = {
        ...optionsConfig[index],
        props: {
          ...optionsConfig[index].props,
          ...radioProps.props
        }
      }
    }
    return h(component, radioProps, item.label)
  })

  const radioGroup = h(RadioGroup, {
    props: {
      ...renderOptions.props,
      value: effectiveValue(value) ? value : renderOptions.props.value
    },
    on: {
      ...on,
      input: (val) => {
        set(fm.form, filed, val)
        typeof on.input === 'function' && on.input(val)
      }
    }
  }, radios)

  return radioGroup
}
