import { Col, Form, Row } from 'element-ui'

import {
  initForm,
  setFormData,
  handlerFormItem,
  createItem,
  createItemChildren
} from './form-utils'
import {copyDeep, isHidden, valueEqual} from '../../utils';
import { searchConfigOption } from '../../utils/handlerConfig';
import { createSearchBtn } from './built-in/search';
import { handlerHeadline } from "./built-in/headline";

export default {
  name: 'LxForm',
  components: {},
  props: {
    isEdit: {
      type: Boolean,
      default: true
    },
    config: {
      type: Array,
      default () {
        return []
      }
    },
    size: {
      type: String,
      default: ''
    },
    value: {
      type: Object,
      default () {
        return {}
      }
    },
    rules: {
      type: Object,
      default () {
        return {}
      }
    },
    formAttr: {
      type: Object,
      default () {
        return {}
      }
    },
    allFormItemStyle: {
      type: Object,
      default () {
        return {}
      }
    },
    allControlStyle: {
      type: Object,
      default () {
        return {}
      }
    },
    defaultBtnShow: {
      type: Boolean,
      // default: '100%'
      default: true
    },
    valid: {
      type: Boolean,
      default: false
    },
    submitBtnText: {
      type: String,
      default: '提交'
    },
    resetBtnText: {
      type: String,
      default: '重置'
    }
  },
  data () {
    return {
      $h: this.$createElement,
      form: {},
      emptyForm: {}
    }
  },
  watch: {
    form: {
      handler (val) {
        this.$emit('input', val)
      },
      deep: true
    },
    config: {
      handler (val) {
        this.form = initForm(this.form, val, this.emptyForm)
      },
      deep: true,
      immediate: true
    },
    value: {
      handler (val) {
        if (valueEqual(this.form, val)) {
          return
        }
        setFormData(this, this.form, val)
      },
      deep: true,
      immediate: true
    }
  },
  methods: {
    searchConfig ({ key, value }) {
      return searchConfigOption(this.config, { key, value })
    },
    getForm () {
      const res = {
        data: this.form,
        formEle: this.$refs.ruleForm
      }
      return res
    },
    validate(callback) {
      return this.$refs.ruleForm.validate(callback)
    },
    reset () {
      this.$refs.ruleForm.resetFields()
    },
    clearValidate () {
      this.$refs.ruleForm.clearValidate()
    },
    validateField () {
      this.$refs.ruleForm.validateField()
    },
    clear () {
      this.form = copyDeep(this.emptyForm)
      setTimeout(() => {
        this.$refs.ruleForm && this.$refs.ruleForm.clearValidate()
      })
    }
  },
  render (h) {
    if (!this.config.length) {
      return
    }
    const formContent = createFormContent.call(this, h)
    const formDom = h(Form,
      {
        ref: 'ruleForm',
        props: {
          model: this.form,
          ...this.formAttr,
          rules: this.rules
        },
        nativeOn: {
          // 监听键盘事件
          keyup: (event) => {
            this.$emit('keyup', event, this.form)
          }
        }
      },
      formContent
    )
    return h(Row, [formDom])
  },
  created () {
  },
  mounted () {
  }
}

function createFormContent (h) {
  let formItems = []
  const config = this.config
  const len = config.length
  for (let i = 0; i < len; i++) {
    const item = config[i]
    const {
      headShow = false,
      children = [], // 表单子项，可多个
      hidden = false, // 隐藏，或者不渲染该组件
      type = '', // 表单组件类型，具体参考 ./components.js
      filed = '', // 字段名称，用于初始化form对象
      props = {}, // 组件 prop
      style = {}, // 接受一个字符串、对象，或对象组成的数组
      classList = {}, // 与 `v-bind:class` 的 API 相同，接受一个字符串、对象或字符串和对象组成的数组
      on: itemOn = {}, // 事件监听器在 `on` 内，但不再支持如 `v-on:keyup.enter` 这样的修饰器。需要在处理函数中手动检查 keyCode。
      attrs = {}, // 普通的 HTML attribute
      domProps = {}, // DOM property
      nativeOn = {}, // // 仅用于组件，用于监听原生事件，而不是组件内部使用，`vm.$emit` 触发的事件。
      directives = [], // 自定义指令。注意，你无法对 `binding` 中的 `oldValue`，赋值，因为 Vue 已经自动为你进行了同步。
      scopedSlots = {}, // 作用域插槽的格式为{ name: props => VNode | Array<VNode> }
      slot = '', // 如果组件是其它组件的子组件，需为插槽指定名称
      key = '', // 其它特殊顶层 property
      ref = '', // 其它特殊顶层 property
      refInFor = true, // 如果你在渲染函数中给多个元素都应用了相同的 ref 名，那么 `$refs.myRef` 会变成一个数组。
      render, // 自定义，优先级顺序 1
      formItemRender // 在formItem中自定义，优先级顺序 2
    } = item

    if (isHidden(hidden, this.form, this.config)) {
      continue
    }

    if (headShow) {
      formItems.push(handlerHeadline(h, item))
      continue
    }

    if (children.length) {
      formItems.push(createItemChildren(this, item))
    } else {
      formItems.push(createItem(this, item))
    }
  }

  this.$scopedSlots.default && formItems.push(this.$scopedSlots.default())

  if (this.$scopedSlots.search) {
    formItems.push(h(Col, {
      style: {
        paddingLeft: this.formAttr.labelWidth || ''
      }
    }, [this.$scopedSlots.search({ form: this.form })]))
  } else {
    if (this.defaultBtnShow && this.isEdit) {
      const searchBtn = createSearchBtn.call(this, h)
      formItems.push(handlerFormItem({}, [searchBtn], this))
    }
  }
  return formItems
}
