import { Menu, MenuItem, Submenu } from 'element-ui'

import { getValue } from "../../utils";
import './style.css'
export default {
  name: 'LxMenu',
  props: {
    defaultActive: {
      type: [String, Number],
      default: ''
    },
    autoJump: {
      type: Boolean,
      default: true
    },
    collapse: {
      type: Boolean,
      default: false
    },
    // 数据
    data: {
      type: Array,
      default () {
        return []
      }
    },
    // 自定义filed
    nodeProps: {
      type: Object,
      default () {
        return {
          title: '',
          index: '',
          children: ''
        }
      }
    },
    // el-menu 的 props
    menuProps: {
      type: Object,
      default () {
        return {}
      }
    },
    height: {
      type: String,
      default: ''
    }
  },
  data () {
    return {
    }
  },
  created () {
  },
  mounted () {
  },
  methods: {
    createMenu () {
      const h = this.$createElement
      const menuChild = this.createMenuChild(h, this.data)
      return h(
        Menu,
        {
          props: {
            ...this.menuProps,
            defaultActive: this.defaultActive.toString(),
            collapse: this.collapse
          },
          class: ['action__linear'],
          style: {
            height: this.height
          },
          on: {
            select: (index, indexPath) => {
              this.$emit('select', index, indexPath)
            }
          }
        },
        [menuChild]
      )
    },
    createMenuChild (h, data) {
      if (!Array.isArray(data)) {
        return []
      }
      const items = []
      data.forEach((item) => {
        if (getValue(item, this.nodeProps.hidden || 'hidden')) {
          return
        }
        const children = getValue(item, this.nodeProps.children) || item.children
        if (children && children.length) {
          items.push(this.createSubMenu(h, children, item))
        } else {
          items.push(this.createMenuItem(h, item))
        }
      })
      return items
    },
    createMenuItem (h, data) {
      const { title, index, path, icon } = this.handlerConfig(data)

      let aHref = path
      if (!(/^(http)(s)?/.test(aHref))) {
        aHref = (new RegExp(/^[#]/)).test(aHref) ? aHref.substring(1) : aHref
        aHref = (new RegExp(/^\//).test(aHref) ? aHref : '/' + aHref)
      }

      const menuItemConfig = {
        props: {
          index: index
        },
        attrs: {
          'data-index': index
        }
      }

      // 如果有标签
      let iconVN = ''
      if (icon) {
        const iconName = icon || ''
        iconVN = [
          h('i', { class: [iconName] }),
        ]
      }

      const contentVN = [
        iconVN,
        h('span', {
          attrs: {title},
          class: ['menu__nav__label']
        }, title)
      ]
      const aTagVN = h('a', {
          class: ['menu_nav__a'],
          attrs: {
            href: aHref
          },
          on: {
            click: (e) => {
              this.$emit('click', data)
              if (this.autoJump) {
                if (!(/^(http)(s)?/.test(aHref))) {
                  this.$router.push(aHref)
                  e.preventDefault()
                }
              } else {
                e.preventDefault()
              }
            }
          }
        }, contentVN)

      return h(MenuItem, menuItemConfig, [aTagVN])
    },
    createSubMenu (h, children, data) {
      const { title, index, icon } = this.handlerConfig(data)

      let childList = this.createMenuChild(h, children)

      if (title) {
        const titleContent = [h('span', title)]
        // 如果有标签
        if (icon) {
          titleContent.unshift(h('i', { class: [icon] }))
        }
        const subMenuTitle = h('div', { slot: 'title' }, titleContent)

        childList.unshift(subMenuTitle)
      }

      return h(Submenu, {
        attrs: {
          'data-index': index
        },
        props: {
          index: index
        }
      }, childList)
    },
    handlerConfig (data) {
      const { children = [], path = '', title = '', index = '', icon = '' } = this.nodeProps

      const _title = getValue(data, title) || this.handlerPropFunc(title, data) || data.title
      const _index = getValue(data, index) || this.handlerPropFunc(index, data) || data.index
      const _children = getValue(data, children) || this.handlerPropFunc(children, data) || data.children
      const _path = getValue(data, path) || this.handlerPropFunc(path, data) || data.path
      const _icon = getValue(data, icon) || this.handlerPropFunc(icon, data) || data.icon
      return {
        index: _index,
        path: _path,
        title: _title,
        children: _children,
        icon: _icon
      }
    },
    handlerPropFunc (key, data) {
      if (typeof key === 'function') {
        return key(data)
      }
      return ''
    }
  },
  render () {
    const menu = this.createMenu()
    return (
      menu
    )
  }
}
