import {copyDeep, valueEqual} from '../../../utils';
import {initHeadCheckboxStatus} from './handler-selection';

export default {
  props: {
    selected: {
      type: Array,
      default () {
        return []
      }
    },
    selectAll: {
      type: Boolean,
      default: false
    }
  },
  data () {
    return {
      tableSelected: [],
      prevTableSelected: [],
      allSelection: {
        indeterminate: false,
        checked: false,
        disabled: false
      },
      allSelectionStatus: false, // indeterminate
      allSelectionChecked: false
    }
  },
  watch: {
    selected: {
      handler (val) {
        if (valueEqual(val, this.prevTableSelected)) {
          return
        }
        this.prevTableSelected = copyDeep(val)
        this.initSelected()
      },
      deep: true
    },
    selectAll: {
      handler (val) {
        this.switchSelectionAll(val)
      },
      immediate: true
    },
    tableSelected: {
      handler (val) {
        this.emitSelected()
        initHeadCheckboxStatus.call(this, val, this.tableData)
      },
      deep: true
    }
  },
  methods: {
    /**
     * 初始化表格数据
     * */
    initTable(data) {
      if (!data || data.length < 1) {
        this.allSelection.indeterminate = false
        this.allSelection.checked = false
        this.allSelection.disabled = true
      }
      this.allSelection.disabled = false
      data.forEach((item) => {
        item.__checked__ = false
      })
      this.tableData = copyDeep(data)
    },
    /**
     * 初始化选择状态
     * */
    initSelected() {
      const primaryKey = this.primaryKey
      const selected = this.selected
      this.tableSelected = []
      this.allSelection.indeterminate = false
      this.allSelection.checked = false
      this.tableData.forEach((item) => {
        item.__checked__ = selected.indexOf(item[primaryKey]) !== -1
        if (item.__checked__) {
          this.tableSelected.push(item)
        }
      })
    },
    // 设置所有复选框的勾选状态
    switchSelectionAll(val) {
      this.tableData.forEach((item) => {
        item.__checked__ = val
      })
      this.tableSelected = val ? copyDeep(this.tableData) : []
    },
    emitSelected() {
      this.$emit('select-change', this.tableSelected)
    },
  }
}
