import {TableColumn, Table} from "element-ui";
import { components } from './config'
import { isArray } from '../../utils';

import { createCheckbox, handlerCheckboxHead } from './selection/handler-selection'
export function createTable (h, data, columns, config) {
  const {
    props,
    attrs,
    classList,
    style,
    on
  } = config
  console.log('config', config)
  const options = {
    ref: 'lxTable',
    props: {
      border: true,
      ...props,
      data
    },
    attrs: {
      ...config
    },
    class: {
      ...classList
    },
    style: {
      width: '100%',
      ...style
    },
    on: {
      ...on
    },
    directives: [
      {
        name: 'loading',
        value: this.loading
      }
    ]
  }
  const columnsVN = cColumns.call(this, h, columns)

  return h(Table, options, columnsVN)
}

export function cColumns (h, columns = []) {
  const columnsVN = []
  const len = columns.length

  for (let i = 0; i < len; i++) {
    if (isArray(columns[i].children) && columns[i].children.length) {
      columnsVN.push(handlerTDItem(h, columns[i], cColumns(h, columns[i].children)))
    } else {
      columnsVN.push(createColumns.call(this, h, columns[i]), i)
    }
  }

  return columnsVN
}

function createColumns (h, item, index) {
  const {
    render,
    type, // 简单处理的数据类型
    selection = false, // 是否启用复选框
    imageStyle = {}, // 如果type === 'image'
    enumLabelKey = '', // 当 type = enum 的时候，以此属性的值为options里的某个字段名来 “显示”
    enumKey = '' // 当 type = enum 的时候，以此属性的值为options里的某个字段名来 “取值” 匹配
  } = item

  if (typeof render === 'function') {
    return handlerTDItem(h, item, (e) => render(h, e, item))
  }
  if (type && components[type]) {
    return handlerTDItem(h, item, components[type].call(this, h, item))
  }
  if (selection) {
    const newItem = handlerCheckboxHead.call(this, h, item)
    const tdVN = createCheckbox.call(this, h, item)
    return handlerTDItem(h, newItem, tdVN)
  }

  return handlerTDItem(h, item)
}

/**
 * @param h createElement
 * @param columnItem options配置
 * @param VNode 一个虚拟DOM集合
 * @return 返回一个 columnVNode
 * */
export function handlerTDItem (h, columnItem, VNode = []) {
  let childrenVNode = VNode
  if (!Array.isArray(childrenVNode)) {
    childrenVNode = [childrenVNode]
  }

  const {
    filed,
    props,
    attrs,
    classList,
    style,
    itemOn,
    scopedSlots
  } = columnItem
  return h(TableColumn, {
    props: {
      ...columnItem,
      ...props,
      prop: filed
    },
    attrs: {
      ...attrs
    },
    class: {
      ...classList
    },
    style: {
      ...style
    },
    on: {
      ...itemOn
    },
    scopedSlots: {
      ...scopedSlots
    }
  }, childrenVNode)
}
